/*
 * Copyright (C) 2001 TAKIZAWA Takashi <taki@cyber.email.ne.jp>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "strerr.h"
#include "exit.h"
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#define FATAL "vida-maildirmake: fatal: "

static void usage(void)
{
  strerr_die1x(100,"Usage: vida-maildirmake virtualuser maildir\n");
}

static char *getpwdir(void)
{
  uid_t pwuid;
  struct passwd *pw;

  pwuid = getuid();
  if (!pwuid)
    strerr_die2x(100,FATAL,"you must change user to the owner of domain");
  pw = getpwuid(pwuid);
  if (!pw)
    strerr_die2x(111,FATAL,"unable to read /etc/passwd");
  return pw->pw_dir;
}

static void maildirmake(char *homedir, char *user, char *maildir)
{
  if (chdir(homedir) == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ",homedir,": ");
  if (mkdir(user,0700) == -1)
    strerr_die4sys(111,FATAL,"unable to mkdir ",user,": ");
  if (chdir(user) == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ",user,": ");
  if (mkdir(maildir,0700) == -1)
    strerr_die4sys(111,FATAL,"unable to mkdir ",maildir,": ");
  if (chdir(maildir) == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ",maildir,": ");
  if (mkdir("tmp",0700) == -1)
    strerr_die4sys(111,FATAL,"unable to mkdir ",maildir,"/tmp: ");
  if (mkdir("new",0700) == -1)
    strerr_die4sys(111,FATAL,"unable to mkdir ",maildir,"/new: ");
  if (mkdir("cur",0700) == -1)
    strerr_die4sys(111,FATAL,"unable to mkdir ",maildir,"/cur: ");
}

int main(int argc,char **argv)
{
  char *homedir;

  umask(077);
  if(!argv[1] || !argv[2])
    usage();

  homedir = getpwdir();
  maildirmake(homedir,argv[1],argv[2]);

  _exit(0);
}
